package org.bidouille.geigerusb;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Locale;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class Counter extends JFrame {
    private static final double CPS_PER_MRH = 25; // Calibration value in cps/mR/h
    private static final NumberFormat format = new DecimalFormat( "#,##0.00" );
    private JLabel cpmLabel;
    private JLabel doseRateLabel;
    private JLabel doseLabel;
    private int count;
    private int[] history = new int[60];
    private int historyPtr;
    private int historySize;

    public Counter() {
        super( "Кварц ДРСБ-01" );
        JPanel mainPanel = new JPanel( new GridLayout( 3, 2 ) );
        add( mainPanel );
        mainPanel.add( createLabel( "Counts per minute :" ) );
        cpmLabel = createLabel( "-" );
        mainPanel.add( cpmLabel );
        mainPanel.add( createLabel( "Dose rate (µR/h) :" ) );
        doseRateLabel = createLabel( "-" );
        mainPanel.add( doseRateLabel );
        mainPanel.add( createLabel( "Total dose (µR) :" ) );
        doseLabel = createLabel( "-" );
        mainPanel.add( doseLabel );

        setPreferredSize( new Dimension( 440, 120 ) );
    }

    public void start() {
        Executors.newSingleThreadScheduledExecutor().scheduleAtFixedRate( new UpdateDisplayTask(), 1, 1, TimeUnit.SECONDS );
    }

    public void setCount( int count ) {
        this.count = count;
    }

    private JLabel createLabel( String text ) {
        JLabel l1 = new JLabel( text );
        l1.setFont( l1.getFont().deriveFont( 22.0f ) );
        l1.setForeground( Color.red );
        return l1;
    }

    private class UpdateDisplayTask implements Runnable {
        @Override
        public void run() {
            historyPtr = (historyPtr + 1) % 60;
            historySize = Math.min( historySize + 1, 60 );
            int prevCount = history[historyPtr];
            history[historyPtr] = count;
            double cpm = (count - prevCount) * 60.0 / historySize;
            cpmLabel.setText( String.valueOf( Math.round( cpm ) ) );
            doseRateLabel.setText( format.format( 1000.0 * (cpm / 60.0) / CPS_PER_MRH ) );
            doseLabel.setText( format.format( 1000.0 * (count / CPS_PER_MRH) / 3600.0 ) );
        }
    }

    public static void main( String[] args ) throws Exception {
        Counter counter = new Counter();
        counter.pack();
        counter.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        counter.setVisible( true );

        communication com = waitForDevice();

        counter.start();
        int oldCount = -1;
        SimpleDateFormat format = new SimpleDateFormat( "HH:mm:ss.SSS" );
        while( true ) {
            int count = getCurrentCount( com );
            if( count != oldCount ) {
                System.out.println( format.format( new Date() ) + " " + count );
                oldCount = count;
                counter.setCount( count );
            }
        }

    }

    private static int getCurrentCount( communication com ) {
        byte[] report = com.GetInputReport();
        // byte[0] is report Id.
        int count = (0xFF & report[2]) << 8 | (0xFF & report[1]);
        return count;
    }

    private static communication waitForDevice() throws InterruptedException {
        communication com = new communication();
        com.SetVendorID( (short) 0x16c0 );
        com.SetProductID( (short) 0x03ec );
        System.out.println( "Looking for device ..." );
        boolean success = com.getHIDHandle();
        while( !success ) {
            Thread.sleep( 1000 );
            success = com.getHIDHandle();
        }
        System.out.println( "Device found." );
        return com;
    }
}
